@@ -40,6 +40,12 @@ from laddr.core import (
 from fastapi import FastAPI, HTTPException, WebSocket, WebSocketDisconnect, Depends, Request
 from fastapi.middleware.cors import CORSMiddleware
 from pydantic import BaseModel
+import jwt
+from datetime import datetime, timedelta
+from slowapi import Limiter, _rate_limit_exceeded_handler
+from slowapi.util import get_remote_address
+from slowapi.errors import RateLimitExceeded
+import os
 
 
 # Request/Response models
@@ -132,6 +138,9 @@ async def lifespan(app: FastAPI):
     logger.info("Laddr API server started")
     logger.info(f"Database: {config.database_url}")
     logger.info(f"Queue: {config.queue_backend}")
+    
+    # Check production security settings
+    _check_production_security()
 
     yield
 
@@ -209,6 +218,11 @@ app = FastAPI(
     lifespan=lifespan,
 )
 
+# Initialize rate limiter
+limiter = Limiter(key_func=get_remote_address)
+app.state.limiter = limiter
+app.add_exception_handler(RateLimitExceeded, _rate_limit_exceeded_handler)
+
 # Add CORS middleware
 app.add_middleware(
     CORSMiddleware,
@@ -222,6 +236,84 @@ app.add_middleware(
     allow_headers=["*"],
 )
 
+# Production validation: Warn if LADDR_API_KEY is not set in production
+def _check_production_security():
+    """Check if production security settings are configured."""
+    is_production = os.getenv("ENVIRONMENT", "").lower() in ("production", "prod")
+    if is_production and not config.laddr_api_key:
+        logger.warning(
+            "⚠️  SECURITY WARNING: LADDR_API_KEY is not set in production environment. "
+            "All API routes are publicly accessible. Set LADDR_API_KEY to protect your API."
+        )
+
+
+# Dashboard Authentication Models
+class LoginRequest(BaseModel):
+    """Dashboard login request."""
+    username: str
+    password: str
+
+
+class LoginResponse(BaseModel):
+    """Dashboard login response."""
+    token: str
+    username: str
+    expires_in: int  # seconds
+
+
+# JWT Configuration
+JWT_SECRET_KEY = os.getenv("JWT_SECRET_KEY", "laddr-dashboard-secret-change-in-production")
+JWT_ALGORITHM = "HS256"
+JWT_EXPIRATION_HOURS = 24 * 7  # 7 days
+
+
+def _parse_dash_users() -> dict[str, str]:
+    """Parse dashboard user credentials from config."""
+    users = {}
+    if config.laddr_dash_users:
+        for pair in config.laddr_dash_users.split(","):
+            parts = pair.strip().split(":")
+            if len(parts) == 2:
+                username, password = parts[0].strip(), parts[1].strip()
+                if username and password:
+                    users[username] = password
+    return users
+
+
+def _create_jwt_token(username: str) -> str:
+    """Create a JWT token for the authenticated user."""
+    payload = {
+        "username": username,
+        "exp": datetime.utcnow() + timedelta(hours=JWT_EXPIRATION_HOURS),
+        "iat": datetime.utcnow(),
+    }
+    return jwt.encode(payload, JWT_SECRET_KEY, algorithm=JWT_ALGORITHM)
+
+
+def _verify_jwt_token(token: str) -> dict | None:
+    """Verify and decode a JWT token."""
+    try:
+        payload = jwt.decode(token, JWT_SECRET_KEY, algorithms=[JWT_ALGORITHM])
+        return payload
+    except jwt.ExpiredSignatureError:
+        return None
+    except jwt.InvalidTokenError:
+        return None
+
+
+async def verify_dashboard_session(request: Request) -> str:
+    """Verify dashboard session token from Authorization header."""
+    auth_header = request.headers.get("Authorization", "")
+    if not auth_header.startswith("Bearer "):
+        raise HTTPException(status_code=401, detail="Missing or invalid authorization header")
+    
+    token = auth_header[7:].strip()
+    payload = _verify_jwt_token(token)
+    if not payload:
+        raise HTTPException(status_code=401, detail="Invalid or expired token")
+    
+    return payload.get("username", "")
+
 
 # API Key Authentication Dependency
 def verify_api_key(request: Request) -> None:
@@ -294,6 +386,48 @@ async def root():
     }
 
 
+@app.post("/api/auth/login")
+@limiter.limit("5/minute")  # Rate limit login attempts
+async def dashboard_login(request: Request, login_req: LoginRequest):
+    """
+    Dashboard login endpoint - server-side authentication.
+    Validates credentials against LADDR_DASH_USERS and returns JWT token.
+    """
+    users = _parse_dash_users()
+    
+    # If no users configured, allow any credentials (development mode)
+    if not users:
+        logger.warning("No dashboard users configured. Allowing any credentials (development mode).")
+        token = _create_jwt_token(login_req.username)
+        return LoginResponse(
+            token=token,
+            username=login_req.username,
+            expires_in=JWT_EXPIRATION_HOURS * 3600
+        )
+    
+    # Validate credentials
+    if login_req.username not in users:
+        raise HTTPException(status_code=401, detail="Invalid username or password")
+    
+    if users[login_req.username] != login_req.password:
+        raise HTTPException(status_code=401, detail="Invalid username or password")
+    
+    # Create JWT token
+    token = _create_jwt_token(login_req.username)
+    
+    return LoginResponse(
+        token=token,
+        username=login_req.username,
+        expires_in=JWT_EXPIRATION_HOURS * 3600
+    )
+
+
+@app.get("/api/auth/me")
+async def dashboard_me(username: str = Depends(verify_dashboard_session)):
+    """Get current authenticated user information."""
+    return {"username": username, "authenticated": True}
+
+
 @app.get("/api/health")
 async def health():
     """Return API health status with system components."""
@@ -323,7 +457,8 @@ async def health():
 
 
 @app.post("/api/jobs")
-async def submit_job(request: SubmitJobRequest, _: None = require_api_key):
+@limiter.limit("30/minute")  # Rate limit job submissions
+async def submit_job(request: Request, job_req: SubmitJobRequest, _: None = require_api_key):
     """
     Submit a new job for execution.
 
@@ -337,8 +472,8 @@ async def submit_job(request: SubmitJobRequest, _: None = require_api_key):
     """
     try:
         result = await run_agent(
-            agent_name=request.pipeline_name,
-            inputs=request.inputs,
+            agent_name=job_req.pipeline_name,
+            inputs=job_req.inputs,
             env_config=config
         )
 
@@ -461,7 +596,8 @@ async def replay_job(job_id: str, request: ReplayJobRequest, _: None = require_a
 # --- New Prompt Endpoints (preferred terminology) ---
 
 @app.post("/api/prompts")
-async def submit_prompt(request: SubmitPromptRequest, _: None = require_api_key):
+@limiter.limit("30/minute")  # Rate limit prompt submissions
+async def submit_prompt(request: Request, prompt_req: SubmitPromptRequest, _: None = require_api_key):
     """
     Submit a new prompt execution (non-blocking).
 
@@ -472,7 +608,7 @@ async def submit_prompt(request: SubmitPromptRequest, _: None = require_api_key)
     """
     try:
         # Create prompt record first and mark as running
-        prompt_id = database.create_prompt(None, request.prompt_name, request.inputs)
+        prompt_id = database.create_prompt(None, prompt_req.prompt_name, prompt_req.inputs)
         database.update_prompt_status(prompt_id, "running")
 
         async def _run_in_background(pid: str):
@@ -480,8 +616,8 @@ async def submit_prompt(request: SubmitPromptRequest, _: None = require_api_key)
                 runner = AgentRunner(env_config=config)
                 
                 # Sequential mode: run multiple agents in order
-                if request.mode == "sequential" and request.agents and len(request.agents) > 1:
-                    result = await _run_sequential_chain(runner, request.agents, request.inputs, pid)
+                if prompt_req.mode == "sequential" and prompt_req.agents and len(prompt_req.agents) > 1:
+                    result = await _run_sequential_chain(runner, prompt_req.agents, prompt_req.inputs, pid)
                 else:
                     # Single agent mode (default)
                     # For single-mode prompt executions initiated from the API UI,
@@ -489,15 +625,15 @@ async def submit_prompt(request: SubmitPromptRequest, _: None = require_api_key)
                     # cross-agent delegation (e.g., researcher delegating to coordinator).
                     # However, allow delegation if the user is directly prompting the
                     # coordinator, since delegation is the coordinator's primary function.
-                    inputs_for_run = dict(request.inputs or {})
+                    inputs_for_run = dict(prompt_req.inputs or {})
                     
                     # Only disable delegation if NOT prompting the coordinator
-                    if request.prompt_name != "coordinator":
+                    if prompt_req.prompt_name != "coordinator":
                         inputs_for_run["_allow_delegation"] = False
 
                     result = await runner.run(
                         inputs=inputs_for_run,
-                        agent_name=request.prompt_name,
+                        agent_name=prompt_req.prompt_name,
                         job_id=pid,
                     )
 
