@@ -59,6 +59,23 @@ class PromptExecution(Base):
     completed_at = Column(DateTime, nullable=True)
 
 
+class Batch(Base):
+    """Batch operation record for tracking parallel task execution."""
+
+    __tablename__ = "batches"
+
+    batch_id = Column(String(36), primary_key=True)  # Unique ID for the batch
+    agent_name = Column(String(255), nullable=False)  # Primary agent (evaluator)
+    status = Column(String(50), default="running")  # running, completed, failed, submitted
+    task_count = Column(Integer, default=0)  # Number of tasks in batch
+    job_ids = Column(JSON, nullable=True)  # List of job_ids for individual tasks in this batch
+    task_ids = Column(JSON, nullable=True)  # List of task_ids for this batch
+    inputs = Column(JSON, nullable=False)  # Original batch request
+    outputs = Column(JSON, nullable=True)  # Final aggregated results
+    created_at = Column(DateTime, default=datetime.utcnow)
+    completed_at = Column(DateTime, nullable=True)
+
+
 class Trace(Base):
     """Trace event for observability with hierarchical span support."""
 
@@ -289,10 +306,17 @@ class DatabaseService:
                 for prompt in prompts
             ]
 
-    def get_job_traces(self, job_id: str) -> list[dict]:
-        """Get all traces for a job."""
+    def get_job_traces(self, job_ids: str | list[str]) -> list[dict]:
+        """Get all traces for one or more jobs."""
+        if isinstance(job_ids, str):
+            job_ids = [job_ids]
         with self.get_session() as session:
-            traces = session.query(Trace).filter_by(job_id=job_id).order_by(Trace.timestamp).all()
+            traces = (
+                session.query(Trace)
+                .filter(Trace.job_id.in_(job_ids))
+                .order_by(Trace.timestamp)
+                .all()
+            )
 
             return [
                 {
@@ -558,3 +582,104 @@ class DatabaseService:
                 "total_tokens": total,
                 "by_model": list(breakdown.values()),
             }
+
+    # Batch management methods
+    def create_batch(
+        self,
+        batch_id: str,
+        agent_name: str,
+        task_count: int,
+        job_ids: list[str],
+        task_ids: list[str],
+        inputs: dict,
+    ) -> None:
+        """Create a new batch record."""
+        with self.get_session() as session:
+            batch = Batch(
+                batch_id=batch_id,
+                agent_name=agent_name,
+                status="running",
+                task_count=task_count,
+                job_ids=job_ids,
+                task_ids=task_ids,
+                inputs=inputs,
+            )
+            session.add(batch)
+
+    def get_batch(self, batch_id: str) -> dict | None:
+        """Get batch by ID."""
+        with self.get_session() as session:
+            batch = session.query(Batch).filter_by(batch_id=batch_id).first()
+            if not batch:
+                return None
+            return {
+                "batch_id": batch.batch_id,
+                "agent_name": batch.agent_name,
+                "status": batch.status,
+                "task_count": batch.task_count,
+                "job_ids": batch.job_ids or [],
+                "task_ids": batch.task_ids or [],
+                "inputs": batch.inputs,
+                "outputs": batch.outputs,
+                "created_at": _iso_z(batch.created_at),
+                "completed_at": _iso_z(batch.completed_at),
+            }
+
+    def update_batch_status(
+        self,
+        batch_id: str,
+        status: str,
+        outputs: dict | None = None,
+        completed_tasks: int | None = None,
+        failed_tasks: int | None = None,
+    ) -> None:
+        """Update batch status and optionally outputs."""
+        with self.get_session() as session:
+            batch = session.query(Batch).filter_by(batch_id=batch_id).first()
+            if not batch:
+                return
+            batch.status = status
+            if outputs is not None:
+                batch.outputs = outputs
+            if status in ("completed", "failed") and batch.completed_at is None:
+                batch.completed_at = datetime.utcnow()
+
+    def add_tasks_to_batch(
+        self,
+        batch_id: str,
+        new_job_ids: list[str],
+        new_task_ids: list[str],
+    ) -> None:
+        """Add more tasks to an existing batch."""
+        with self.get_session() as session:
+            batch = session.query(Batch).filter_by(batch_id=batch_id).first()
+            if not batch:
+                return
+            existing_job_ids = batch.job_ids or []
+            existing_task_ids = batch.task_ids or []
+            batch.job_ids = existing_job_ids + new_job_ids
+            batch.task_ids = existing_task_ids + new_task_ids
+            batch.task_count = len(batch.job_ids)
+
+    def list_batches(self, limit: int = 50) -> list[dict]:
+        """List recent batches."""
+        with self.get_session() as session:
+            batches = (
+                session.query(Batch)
+                .order_by(Batch.created_at.desc())
+                .limit(limit)
+                .all()
+            )
+            return [
+                {
+                    "batch_id": batch.batch_id,
+                    "agent_name": batch.agent_name,
+                    "status": batch.status,
+                    "task_count": batch.task_count,
+                    "job_ids": batch.job_ids or [],
+                    "task_ids": batch.task_ids or [],
+                    "created_at": _iso_z(batch.created_at),
+                    "completed_at": _iso_z(batch.completed_at),
+                }
+                for batch in batches
+            ]
